#----------------------------------------------------------------------
# Name:        chatwin.py
# Purpose:     Jabber interface
#
# Author:      Kristofer Goransson
#
# Created:     2003 Kristofer Goransson
# Copyright:   (c) 2003 
# Licence:     GPL
#---------------------------------------------------------------------- 
import sys, jabber, time, config

class Jabb:
    def __init__(self, parent):
        self.config = config.Config()
        self.main = parent
        self.jids = []
    
    def connect(self):
        consets = self.config.getSettings('general')
        debugv = int(consets['debug'])
        autologon = consets['autologon']
                
        jabbsets = self.config.getSettings('jabber')
        self.server=jabbsets['jabberserver']
        self.agentserver=jabbsets['agentserver']
        self.icqserver=jabbsets['icqserver']
        self.username=jabbsets['username']
        self.passwd=jabbsets['password']
        self.icq=jabbsets['icqnum']

        if debugv == 0:
            debugv=False
            logv=False
        elif debugv == 1:
            debugv=False
            logv=sys.stdout
        elif debugv == 2:
            debugv=True
            logv=sys.stdout
        else:
            self.main.output('Wrong debug value!. Edit config.cfg', 'red')
        self.jabcon = jabber.Client(host=self.server,debug=debugv,log=logv)
        try:
            self.main.output('Connecting... ', 'white')
            self.jabcon.connect()
        except IOError, e:
            self.main.output('Couldn\'t connect: '+str(e), 'red')
            sys.exit(0)
        else:
            self.main.output('Connected to '+self.server, 'green')

            self.jabcon.setMessageHandler(self.messageCB)
            self.jabcon.setPresenceHandler(self.presenceCB)
            self.jabcon.setIqHandler(self.iqCB)
            self.jabcon.setDisconnectHandler(self.disconnCB)

            if autologon == 'yes' or autologon == 'True':
                self.logon()

    def logon(self):
        if self.jabcon.auth(self.username,self.passwd,'default'):
            self.main.output('Logged in with '+self.username, 'green')
        else:
            self.main.output('Error logging in', 'red')
            sys.exit(0)
        self.roster = self.jabcon.requestRoster()
        self.jabcon.sendInitPresence()

    def waitProcess(self):
        while(1):
            self.jabcon.process(1)

    def getIcqServer(self):
        return self.icqserver

    def disconnCB(self, con):
        print self.main.output('Disconnected!', 'red')
        sys.exit(0)

    def messageCB(self, c, msg):
        error = msg.getError()
        mess = msg.getBody()
        """try:                    #fix!!
            print 'Got mess:',mess
        except:
            print 'UnicodeError (%s)'%(msg.getFrom())
        else:"""
        if error:
            self.main.output('Received error messages: '+error, 'red')
        else:
            if mess:
                jid = self.stripJID(msg.getFrom())
                self.main.jabbrequest(jid,mess)#self.stripJID(jid), mess)
            else:
                self.main.output('Received unknown message: '+mess)
    
    def presenceCB(self, con, prs):
        fromjid = self.stripJID(prs.getFrom())
        if prs.getType() == 'error':
            self.main.output('Presence error!', 'warning')

        if prs.getType() == 'subscribe':
            #self.main.output('Received presence %s from %s'%(prs.getType(),prs.getFrom()))
            con.send(jabber.Presence(fromjid, 'subscribed'))    #accept
            self.addJID(fromjid)                                #send subscribe
                
        #when authed??
        if prs.getType() == 'subscribed':
            if self.main.addUser(fromjid):
                self.main.saveUsers()
                self.main.output('%s is now locally added' %fromjid, 'green')
            else:
                self.main.output('User %s already added' %fromjid, 'white')
        self.main.output('Received presence %s from %s'%(prs.getType(),prs.getFrom()))

    def sendMessage(self, who, txt):
        msg = jabber.Message(who, '\n'+txt)
        msg.setType('chat')
        self.jabcon.send(msg)

    def getJIDs(self):
        jids = self.roster.getJIDs()
        return jids

    def getStrJIDs(self):
        jids = self.getJIDs()
        list = []
        for jid in jids:
            list.append(jid.getStripped())
        return list

    def addJID(self, jid):
        self.jabcon.addRosterItem(jid)

    def delJID(self, jid):
        self.jabcon.removeRosterItem(jid)

    def stripJID(self, jid):
        return str(jid.getStripped())

    def iqCB(self, c, iq):
        self.main.output('Received iq %s from %s'%(iq.getType(),iq.getFrom()))

    def getCon(self):
        return self.jabcon

    def sendPres(self, fromjid, type):
        self.jabcon.send(jabber.Presence(fromjid, type))

    def getOnline(self, jid):
        return self.roster.isOnline(jid)

    def getShow(self, jid):
        return self.roster.getShow(jid)

    def unregister(self):
        self.main.output('unregister icq-agent','white')
        reg_iq = jabber.Iq(to = self.icqserver, type='set')
        q = reg_iq.setQuery("jabber:iq:register")
        q.insertTag('remove').putData('')
        self.jabcon.SendAndWaitForResponse(reg_iq)

    def register(self, agent=None):
        self.main.output('agent','white')
        if agent == 'icq':
            #self.agent = 'icq.'
            #self.jabcon.requestAgents()
            self.main.output('Requesting agents...','white')
            ag_iq = jabber.Iq(to = self.agentserver, type='get')
            ag_iq.setQuery("jabber:iq:agents")
            self.jabcon.SendAndWaitForResponse(ag_iq)
            
            #self.jabcon.requestRegInfo('icq')
            self.main.output('Getting register...','white')
            reg_iq = jabber.Iq(to = self.icqserver, type='get')
            reg_iq.setQuery("jabber:iq:register")
            self.jabcon.SendAndWaitForResponse(reg_iq)
        else:
            self.agent = ''
            self.jabcon.requestRegInfo()
        req = self.jabcon.getRegInfo()

        if agent == 'icq':
            reg_info = {'key':req['key'],'username':self.icq,'password':self.passwd}
            infos = ['key','username','password']
            reg_iq = jabber.Iq(to = self.icqserver, type='set')
        else:
            reg_info = {'username':self.username,'password':self.passwd}
            infos = ['username','password']
            reg_iq = jabber.Iq(to = self.server, type='set')
        self.main.output(reg_info,'white')

        #reg_iq = jabber.Iq(to = self.agent+self.server, type='set')
        q = reg_iq.setQuery("jabber:iq:register")
        for info in infos:
            print info
            q.insertTag(info).putData(reg_info[info])
        self.jabcon.SendAndWaitForResponse(reg_iq)

    def disconnect(self):
        self.main.output('Logging out...', 'white')
        self.jabcon.disconnect()
